#!/usr/bin/python

### Load modules in all categories (monitor, mouse, etc.)
try:
	import os, sys
	from keyboard import *
	from device import *
	from arch import *
	from fonts import *
	from misc import *
	from template import Section
	from mouse import detect_mouse_type

except ImportError, e:
	print "Module loading failed with: " + str(e)
	print "Please check that xac was installed correctly"
	sys.exit()

### Abstracts the configuration to allow seperate front/backends
class XAC_Config:
	def __init__(self, version):
		self.version = version
		self.dri = 1
		self.safe = 0
		self.arch = get_arch()
		self.filename = "xorg.conf"

		### "Server Flags" Settings
		self.blank_time = 10
		self.standby_time = 20
		self.suspend_time = 30
		self.off_time = 60
		self.mouse_fail = 0
		self.allow_mode_change = 1

		### Instance each module
		self.mods = Modules()
		for mod in ["extmod", "dbe", "record", "glx", "type1", "freetype"]:
			self.mods.add(mod)

		self.kbds = [XAC_Keyboard()]
		self.mice = [detect_mouse_type(self.arch.get_machine())]
		self.fonts = Fonts()

		### Perform arch specific workarounds
		self.arch.arch_workarounds()

	### Get the arch pointer
	def arch(self):
		return self.arch

	def get_config(self):
		### Try to determine the card
		if self.safe:
			d = self.arch.arch_default_driver()
			c = XAC_Card(d, None, 0, "Unknown - Architecture Default")
			c.monitors.append(XAC_Monitor())
			self.cards = [ c ]
		else:
			self.cards = self.arch.cards

		### Add the DRI module if we need it
		if self.dri:
			for d in self.cards:
				if d.driver.dri:
					self.mods.add("dri")
					break
		else:
			self.mods.rm("dri")

		### Perform arch specific workarounds
		self.arch.arch_workarounds()

		### Generate config by asking each module for its input
		output = "### This file was generated by xac v" + self.version + "\n\n"
		output = output + get_server_layout()
		output = output + self.fonts.get_section()
		output = output + self.mods.get_section()
		### Extensions?
		output = output + get_server_flags(self)
		### Keyboards
		if len(self.kbds) == 0:
			output = output + "### Warning, no keyboards were configured"
		for k in self.kbds:
			output = output + k.get_section()
		### Mice
		if len(self.mice) == 0:
			output = output + "### Warning, no mice were configured"
		for m in self.mice:
			output = output + m.get_section()

		### Videeo Cards and monitors
		if self.arch.warning:
			output = output + "### xac couldn't detect your video card, this is the default\n"
		for d in self.cards:
			output = output + d.get_section()
			for m in d.monitors:
				output = output + m.get_section()
			if self.dri and d.driver.dri:
				self.dri = 0
				output = output + get_dri_section()
		return output

	### Save the config to a file
	def save_config(self):
		OUT = open(self.filename, "w")
		OUT.write(self.get_config())
		OUT.close
