#! /usr/bin/env python

from template import Section

### Detects your mouse
### m is the output of arch.get_machine()
def detect_mouse_type(m = ""):
	### Check /proc/bus/input/devices for synaptics or appletouch	
	input_devs = open("/proc/bus/input/devices", mode="r")	
	lines = input_devs.readlines()
	for l in range(len(lines)):
		line = lines[l].upper()
		### Found a synaptics device
		if line.count("SYNAPTICS") or line.count("APPLETOUCH") or line.count("ALPS"):
			### FIXME Should I stick with mice or go with these?
			for l in range(l, len(lines[l:]) + l):
				line = lines[l].upper()
				found = line.find("EVENT")
				if found != -1:
					return Synaptics(device="/dev/input/" + lines[l][found:found + 6], machine=m)
			### Just in case
			return Synaptics(machine = m)
		### Probe for a serial mouse
		### FIXME

		### Probe for a standard mouse
		if line.count("IMPS"):
			return Standard(protocol="IMPS/2")

	### If we didn't find anything, return a hopefully working default
	return Standard()

### m is the driver name
def get_mouse(driver = "", device=None, id=0, protocol=None, machine = ""):
	if driver == 'mouse': 
		return Standard(device, id, protocol, machine)
	elif driver == 'serial':
		return Serial(device, id, protocol, machine)
	elif driver == 'synaptics':
		return Synaptics(device, id, protocol, machine)
	else:
		return None

### Mouse / Input device
class XAC_Mouse:
	def __init__(self, device=None, id=0, protocol=None, machine=""):
		if device:
			self.device = device
		else:
			self.device = "/dev/input/mice"

		self.id = id
		self.name = "Unknown Mouse"
		if protocol:
			self.protocol = protocol
		else:
			self.protocol = "auto"
		self.machine = machine
		self.driver = 'mouse'

	def get_section(self):
		s = Section()
		s.name('InputDevice')
		s.item('Option\t\t')
		s.add('Identifier\t"Mouse' + str(self.id) + '"')
		s.add('Driver\t\t"' + self.driver + '"')
		s.addi('"Device"\t\t"' + self.device + '"')
		s.addi('"Protocol"\t\t"' + self.protocol +  '"')
		s.add_blank()
		s.add('Set driver options:', 3)
		self.set_driver_options(s)
		s.add_blank()
		s.add('Unset driver options:', 3)
		self.unset_driver_options(s)
		return s.get_section()
	
	def set_driver_options(self, s):
		s.add('No options set!', 3)
	
	def unset_driver_options(self, s):
		s.add('No options unset!', 3)

### Setup Synaptics
class Synaptics(XAC_Mouse):
	def __init__(self, device="/dev/input/mice", id=0, protocol="auto", machine=""):
		XAC_Mouse.__init__(self, device, id, protocol, machine)
		self.driver = "synaptics"
		self.tapping = "Single Tap"
		self.name = "Synaptics/ALPS Touchpad"

	def set_driver_options(self, s):
		s.add('The gsynaptics and ksynaptics applications can help to',3)
		s.add('better configure your trackpad.',3)
		s.add_blank()
		s.addi('"SendCoreEvents"\t"true"')
		if self.tapping == "None":
			s.add('Set this value to enable tapping',3)
			s.addi('"MaxTapTime"\t\t"0"')
		else:
			s.add('Set this to 0 to disable all tapping',3)
			s.addi('"MaxTapTime"\t\t"100"')
			
		s.addi('"HorizScrollDelta"\t"30"')
		s.addi('"VertScrollDelta"\t"30"')
		s.addi('"UpDownScrolling"\t"1"\t# Enables up/down scrolling')
		s.addi('"SHMConfig"\t\t"on"\t# Allows on the fly driver configuration')
		if self.tapping == "Finger Tap":
			s.addi('"TapButton2"\t\t"3"\t# Enables right button 2 finger tapping')
			s.addi('"TapButton3"\t\t"2"\t# Enables middle button 3 finger tapping')
       		elif self.tapping == "Corner Tap":
			s.addi('"RTCornerButton"\t\t"3"')
			s.addi('"LTCornerButton"\t\t"2"')
		else:
			s.add('To Enable right and middle button tapping:', 3)
			s.add('Using finger combinations:', 3)
			s.addi('"TapButton2"\t\t"3"\t# Enables right button 2 finger tapping',1)
			s.addi('"TapButton3"\t\t"2"\t# Enables middle button 3 finger tapping',1)
			s.add('Using the touchpad corners:', 3)
			s.addi('"RTCornerButton"\t\t"3"',1)
			s.addi('"LTCornerButton"\t\t"2"',1)
			
		s.add_blank()
		
		### PowerBook 17"
		if self.machine.count("PowerBook5,9"):
			s.add('This is a configuration for a PowerBook5,9 (17")', 3)
			s.add('You may need to configure these options for better results',3)
			s.addi('"LeftEdge"\t\t"0"')
			s.addi('"RightEdge"\t\t"850"')
			s.addi('"TopEdge"\t\t"0"')
			s.addi('"BottomEdge"\t\t"645"')
			s.addi('"MinSpeed"\t\t"0.4"')
			s.addi('"MaxSpeed"\t\t"1"')
			s.addi('"AccelFactor"\t\t"0.04"')
			s.addi('"FingerLow"\t\t"0"')
			s.addi('"FingerHigh"\t\t"30"')

		### PowerBook 15"
		elif self.machine.count("PowerBook5,8"):
			s.add('This is a configuration for a PowerBook5,8 (15")', 3)
			s.add('You may need to configure these options for better results',3)
			s.addi('"LeftEdge"\t\t"50"')
			s.addi('"RightEdge"\t\t"840"')
			s.addi('"TopEdge"\t\t"30"')
			s.addi('"BottomEdge"\t"320"')
			s.addi('"MinSpeed"\t\t"0.2"')
			s.addi('"MaxSpeed"\t\t"1.5"')
			s.addi('"AccelFactor"\t\t"0.1"')
			s.addi('"FingerLow"\t\t"12"')
			s.addi('"FingerHigh"\t\t"20"')
			
	def unset_driver_options(self, s):
		if self.tapping == 'None':
			s.add('Set this value to enable single finger tapping',3)
			s.addi('"MaxTapMove"\t\t"0"')
		else:
			s.add('Set this to 0 to disable single finger tapping',3)
			s.addi('"MaxTapMove"\t\t"20"',1)
		### Generic driver options
		if not (self.machine.count("PowerBook5,9") or self.machine.count("PowerBook5,8")):
			s.add('You may need to configure these options for',3)
			s.add('a more accurate touchpad',3)
			s.addi('"LeftEdge"\t\t"0"',1)
			s.addi('"RightEdge"\t\t"0"',1)
			s.addi('"TopEdge"\t\t"0"',1)
			s.addi('"BottomEdge"\t\t"0"',1)
			s.addi('"MinSpeed"\t\t"0"',1)
			s.addi('"MaxSpeed"\t\t"0"',1)
			s.addi('"AccelFactor"\t\t"0"',1)
			s.addi('"FingerLow"\t\t"0"',1)
			s.addi('"FingerHigh"\t\t"0"',1)
	
### Setup Standard Mouse
class Standard(XAC_Mouse):
	def __init__(self, device="/dev/input/mice", id=0, protocol="auto", machine=""):
		XAC_Mouse.__init__(self, device, id, protocol, machine)
		self.driver = "mouse"
		self.name = "Generic Mouse"

### Serial Mice
class Serial(XAC_Mouse):
	def __init__(self, device="/dev/input/mice", id=0, protocol="auto", machine=""):
		XAC_Mouse.__init__(self, device, id, protocol, machine)
		self.driver = "serial"
		self.name = "Serial Mouse"
