/* 
 * Parts of this file were part of ddcprobe (which is part of 
 * Xorgautoconfig-0.1.x and 0.2.x.)  I'm not sure as to who 
 * originally wrote this code, but the readme indicates that 
 * it's Nalin Dahyabhai (bigfun@pobox.com).  There was no header
 * but ddcprobe was distibuted with a copy of the GPL version 2
 *
 * Get an edid from the VESA info on x86 using int10
 * Joseph Jezak, 2006
 *
 */

#include <sys/types.h>
#include <sys/io.h>
#include <sys/mman.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include <limits.h>
#include <ctype.h>
#include <Python.h>
#include "lrmi.h"
#include "vbe.h"

/* Check if EDID queries are supported. */
static PyObject * vbe_get_edid_supported()
{
	struct LRMI_regs regs;
	int ret = 0;

	/* Initialize LRMI. */
	if(LRMI_init() == 0) {
		return 0;
	}

	memset(&regs, 0, sizeof(regs));
	regs.eax = 0x4f15;
	regs.ebx = 0x0000;
	regs.es = 0x3000;
	regs.edi = 0x3000;

	/* Do it. */
	iopl(3);
	ioperm(0, 0x400, 1);

	if(LRMI_int(0x10, &regs) == 0) {
		return 0;
	}

	/* Check for successful return. */
	if((regs.eax & 0xff) == 0x4f) {
		/* Supported. */
		ret = 1;
	} else {
		/* Not supported. */
		ret = 0;
	}

	/* Clean up and return. */
	return Py_BuildValue("i", ret);
}

/* Get EDID info. */
static PyObject * vbe_get_edid()
{
	struct LRMI_regs regs;
	unsigned char *mem;
	struct vbe_edid1_info *ret = NULL;
	u_int16_t man;
	PyObject *p;

	/* Initialize LRMI. */
	if(LRMI_init() == 0) {
		return NULL;
	}

	/* Allocate a chunk of memory. */
	mem = LRMI_alloc_real(sizeof(struct vbe_edid1_info));
	if(mem == NULL) {
		return NULL;
	}
	memset(mem, 0, sizeof(struct vbe_edid1_info));

	memset(&regs, 0, sizeof(regs));
	regs.eax = 0x4f15;
	regs.ebx = 0x0001;
	regs.es = ((u_int32_t)mem) >> 4;
	regs.edi = ((u_int32_t)mem) & 0x0f;

	/* Do it. */
	iopl(3);
	ioperm(0, 0x400, 1);

	if(LRMI_int(0x10, &regs) == 0) {
		LRMI_free_real(mem);
		return NULL;
	}

#if 0
	/* Check for successful return. */
	if((regs.eax & 0xffff) != 0x004f) {
		LRMI_free_real(mem);
		return NULL;
	}
#elseif
	/* Check for successful return. */
	if((regs.eax & 0xff) != 0x4f) {
		LRMI_free_real(mem);
		return NULL;
	}
#endif

	/* Get memory for return. */
	ret = malloc(sizeof(struct vbe_edid1_info));
	if(ret == NULL) {
		LRMI_free_real(mem);
		return NULL;
	}

	/* Copy the buffer for return. */
	memcpy(ret, mem, sizeof(struct vbe_edid1_info));

	memcpy(&man, &ret->manufacturer_name, 2);
	man = ntohs(man);
	memcpy(&ret->manufacturer_name, &man, 2);

	LRMI_free_real(mem);
	p = Py_BuildValue("s#", ret, 128);
	
	free(ret);
	return p;
}

static struct PyMethodDef module_methods[] = {
	{"vbe_get_edid", (PyCFunction)vbe_get_edid, METH_NOARGS, "Gets an EDID from the VESA"},
	{"vbe_get_edid_supported", (PyCFunction)vbe_get_edid_supported, METH_NOARGS, "Is getting EDID from VESA supported?"},
	{NULL, NULL, 0, NULL},
};


PyMODINIT_FUNC initvbe(void) {
	PyObject *m;
	m = Py_InitModule3("vbe", module_methods,"Allow VESA EDID info to be accessed on x86");
}
